package javawrapper;

import junit.framework.*;

/**
 * Tests the DataMember class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestDataMember
    extends TestCase {

  public TestDataMember(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Test of first constructor - make sure the label passed gets assigned to
   * both display name and code name
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testDataMember() {
    String sLabel = "test name";
    DataMember oDataMember = new DataMember(sLabel);
    assertEquals(sLabel, oDataMember.GetCodeName());
    assertEquals(sLabel, oDataMember.GetDisplayName());
  }

  /**
   * Test of second constructor
   */
  public void testDataMember1() {
    String sDisplayName = "test display name";
    String sCodeName = "test code name";
    int iType = DataMember.BOOLEAN;
    DataMember oDataMember = null;
    try {
      oDataMember = new DataMember(sDisplayName, sCodeName, iType);
    }
    catch (ModelException oErr) {
      fail("DataMember constructor failed with message " + oErr.sMessage);
    }

    oDataMember.SetCode(6);
    assertEquals(DataMember.BOOLEAN, oDataMember.GetType());
    assertEquals(sDisplayName, oDataMember.GetDisplayName());
    assertEquals(sCodeName, oDataMember.GetCodeName());
    assertEquals(6, oDataMember.GetCode());

    System.out.println("Constructor test succeeded for DataMember.");

    //Exception testing with the constructor
    try {
      oDataMember = null;
      oDataMember = new DataMember(sDisplayName, sCodeName, 25);
      fail("Data member constructor failed to spot bad type code.");
    }
    catch (ModelException oErr) {
      System.out.println(
          "Exception test for constructor test succeeded for DataMember.");
    }
  }

  public void testClone() {
    String sDisplayName = "test display name";
    String sCodeName = "test code name";
    int iType = DataMember.BOOLEAN;
    int iCode = 6;
    DataMember oDataMember = null;
    try {
      oDataMember = new DataMember(sDisplayName, sCodeName, iType);
    }
    catch (ModelException oErr) {
      fail("DataMember constructor failed with message " + oErr.sMessage);
    }
    oDataMember.SetCode(iCode);

    DataMember oClone = (DataMember) oDataMember.clone();
    assertEquals(oDataMember.GetCode(), oClone.GetCode());
    assertEquals(oDataMember.GetCodeName(), oClone.GetCodeName());
    assertEquals(oDataMember.GetDisplayName(), oClone.GetDisplayName());
    assertEquals(oDataMember.GetType(), oClone.GetType());

    //Now make sure it's a deep clone
    oDataMember.SetCode(8);
    assertEquals(iCode, oClone.GetCode());
    oDataMember = null;
    assertEquals(iCode, oClone.GetCode());
    assertEquals(sCodeName, oClone.GetCodeName());
    assertEquals(sDisplayName, oClone.GetDisplayName());
    assertEquals(iType, oClone.GetType());

    System.out.println("Clone test succeeded for DataMember.");

  }

}
