package javawrapper;

import junit.framework.*;
import java.util.*;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 * Tests the PlantingBehavior class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 24, 2004:  Added testing of XML parsing (LEM)
 */
public class TestPlantingBehaviors
    extends TestCase {
  private PlantingBehaviors m_oPlantingBehaviors = null;

  protected void setUp() throws Exception {
    super.setUp();
    m_oPlantingBehaviors = new PlantingBehaviors(null);
  }

  protected void tearDown() throws Exception {
    m_oPlantingBehaviors = null;
    super.tearDown();
  }

  public void testAddPlantingData() throws ModelException {
    Vector oList = null;
    PlantingData oData = null;
    Plot oPlot;

    try {
      oPlot = new Plot(null);
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(100);

      //Null list and harvest data
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(null, oList);

      //First one to be added to a null list
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(1, oList.size());

      //Next item has different plant spacing
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetSpacingDistance(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(2, oList.size());

      //Next item has different timestep
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(2);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(3, oList.size());

      //Next item has different species
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(2);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.AddSpecies(3);
      oData.AddAbundance(3, 4);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(4, oList.size());

      //Next item has different abundance
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 33);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(5, oList.size());

      //This item is the same as the first one except for cells - should be
      //combined with it
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(1);
      oData.AddCell(1, 2, oPlot);
      oData.AddCell(4, 5, oPlot);
      oData.AddCell(7, 4, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(5, oList.size());
      oData = (PlantingData) oList.elementAt(0);
      assertEquals(6, oData.GetNumberOfCells());

      //Now try to add that same one again - nothing should change
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oData.SetTimestep(1);
      oData.AddCell(1, 2, oPlot);
      oData.AddCell(4, 5, oPlot);
      oData.AddCell(7, 4, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetDensity(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(5, oList.size());
      oData = (PlantingData) oList.elementAt(0);
      assertEquals(6, oData.GetNumberOfCells());

      //This item differs from the second item only by plant spacing distance -
      //should be added as unique
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetSpacingDistance(6);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(6, oList.size());

      //This item is the same as the second item - add cells only
      oData = null;
      oData = new PlantingData();
      oData.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(2, 4, oPlot);
      oData.AddCell(6, 4, oPlot);
      oData.AddSpecies(0);
      oData.AddSpecies(2);
      oData.AddAbundance(0, 12);
      oData.AddAbundance(2, 34);
      oData.SetSpacingDistance(4);
      oList = m_oPlantingBehaviors.AddPlantingData(oList, oData);
      assertEquals(6, oList.size());
      oData = (PlantingData) oList.elementAt(1);
      assertEquals(5, oData.GetNumberOfCells());
    }
    catch (ModelException oErr) {
      fail(
          "Normal processing failed for AddPlantingData with message " +
          oErr.sMessage);
    }
  }

  /**
   * Tests parameter file reading.
   */
  public void testReadXML() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLFile1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      TreePopulation oPop = oManager.GetTreePopulation();

      PlantingBehaviors oPlant = oManager.GetPlantingBehaviors();

      //*************************
       //Verify planting initial diam10s
       //*************************
      int i, iNumSpecies = oPop.GetNumberOfSpecies();
      for (i = 0; i < iNumSpecies; i++) {
        float fDiam10 = new Float(String.valueOf(oPlant.mp_fInitialDiam10.
                                                 GetValue().elementAt(i))).
            floatValue();
        assertEquals(fDiam10, (0.1 * i) + 0.1, 0.001);
      }

      //*************************
       //Test each planting event
       //*************************
        assertEquals(2, oPlant.mp_oPlantings.size());

        //Number one
      PlantingData oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(
          0);
      assertEquals(1, oPlantEvent.GetTimestep());
      assertEquals(PlantingBehaviors.GRIDDED, oPlantEvent.GetPlantSpacing());
      assertEquals(3.2, oPlantEvent.GetSpacingDistance(), 0.001);
      assertEquals(1, oPlantEvent.GetNumberOfSpecies());
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(0));
      assertEquals(100,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Subalpine_Fir")),
                   0.001);

      assertEquals(1, oPlantEvent.GetNumberOfCells());
      assertEquals(6, oPlantEvent.GetCell(0).GetX());
      assertEquals(7, oPlantEvent.GetCell(0).GetY());

      //Number two
      oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(1);
      assertEquals(2, oPlantEvent.GetTimestep());
      assertEquals(PlantingBehaviors.RANDOM, oPlantEvent.GetPlantSpacing());
      assertEquals(40, oPlantEvent.GetDensity(), 0.001);
      assertEquals(9, oPlantEvent.GetNumberOfSpecies());
      assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"),
                   oPlantEvent.GetSpecies(0));
      assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"),
                   oPlantEvent.GetSpecies(1));
      assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"),
                   oPlantEvent.GetSpecies(2));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"),
                   oPlantEvent.GetSpecies(3));
      assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"),
                   oPlantEvent.GetSpecies(4));
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(5));
      assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"),
                   oPlantEvent.GetSpecies(6));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"),
                   oPlantEvent.GetSpecies(7));
      assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"),
                   oPlantEvent.GetSpecies(8));

      assertEquals(0,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Western_Redcedar")),
                   0.001);
      assertEquals(10,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Trembling_Aspen")),
                   0.001);
      assertEquals(11,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Subalpine_Fir")),
                   0.001);
      assertEquals(12,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Paper_Birch")),
                   0.001);
      assertEquals(3,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Amabilis_Fir")),
                   0.001);
      assertEquals(16,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Black_Cottonwood")),
                   0.001);
      assertEquals(21,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Lodgepole_Pine")),
                   0.001);
      assertEquals(8,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Western_Hemlock")),
                   0.001);
      assertEquals(19,
                   oPlantEvent.GetAbundance(oPop.GetSpeciesCodeFromName("Hybrid_Spruce")),
                   0.001);

      assertEquals(19, oPlantEvent.GetNumberOfCells());
      assertEquals(8, oPlantEvent.GetCell(0).GetX());
      assertEquals(6, oPlantEvent.GetCell(0).GetY());

      assertEquals(8, oPlantEvent.GetCell(1).GetX());
      assertEquals(7, oPlantEvent.GetCell(1).GetY());

      assertEquals(8, oPlantEvent.GetCell(2).GetX());
      assertEquals(8, oPlantEvent.GetCell(2).GetY());

      assertEquals(8, oPlantEvent.GetCell(3).GetX());
      assertEquals(9, oPlantEvent.GetCell(3).GetY());

      assertEquals(8, oPlantEvent.GetCell(4).GetX());
      assertEquals(10, oPlantEvent.GetCell(4).GetY());

      assertEquals(8, oPlantEvent.GetCell(5).GetX());
      assertEquals(11, oPlantEvent.GetCell(5).GetY());

      assertEquals(8, oPlantEvent.GetCell(6).GetX());
      assertEquals(12, oPlantEvent.GetCell(6).GetY());

      assertEquals(8, oPlantEvent.GetCell(7).GetX());
      assertEquals(13, oPlantEvent.GetCell(7).GetY());

      assertEquals(8, oPlantEvent.GetCell(8).GetX());
      assertEquals(14, oPlantEvent.GetCell(8).GetY());

      assertEquals(8, oPlantEvent.GetCell(9).GetX());
      assertEquals(15, oPlantEvent.GetCell(9).GetY());

      assertEquals(8, oPlantEvent.GetCell(10).GetX());
      assertEquals(16, oPlantEvent.GetCell(10).GetY());

      assertEquals(8, oPlantEvent.GetCell(11).GetX());
      assertEquals(17, oPlantEvent.GetCell(11).GetY());

      assertEquals(8, oPlantEvent.GetCell(12).GetX());
      assertEquals(18, oPlantEvent.GetCell(12).GetY());

      assertEquals(8, oPlantEvent.GetCell(13).GetX());
      assertEquals(19, oPlantEvent.GetCell(13).GetY());

      assertEquals(8, oPlantEvent.GetCell(14).GetX());
      assertEquals(20, oPlantEvent.GetCell(14).GetY());

      assertEquals(8, oPlantEvent.GetCell(15).GetX());
      assertEquals(21, oPlantEvent.GetCell(15).GetY());

      assertEquals(8, oPlantEvent.GetCell(16).GetX());
      assertEquals(22, oPlantEvent.GetCell(16).GetY());

      assertEquals(8, oPlantEvent.GetCell(17).GetX());
      assertEquals(23, oPlantEvent.GetCell(17).GetY());

      assertEquals(8, oPlantEvent.GetCell(18).GetX());
      assertEquals(24, oPlantEvent.GetCell(18).GetY());

      new java.io.File(sFileName).delete();
    }
    catch (ModelException oErr) {
      fail("XML tree map reading failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests change of species.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLFile1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      new java.io.File(sFileName).delete();

      TreePopulation oPop = oManager.GetTreePopulation();
      PlantingBehaviors oPlant = oManager.GetPlantingBehaviors();

      //*************************
       //Verify initial plantings
       //*************************
      assertEquals(2, oPlant.mp_oPlantings.size());

      //Number one
      PlantingData oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(
          0);
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(0));

      //Number two
      oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(1);
      assertEquals(9, oPlantEvent.GetNumberOfSpecies());
      assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"),
                   oPlantEvent.GetSpecies(0));
      assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"),
                   oPlantEvent.GetSpecies(1));
      assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"),
                   oPlantEvent.GetSpecies(2));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"),
                   oPlantEvent.GetSpecies(3));
      assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"),
                   oPlantEvent.GetSpecies(4));
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(5));
      assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"),
                   oPlantEvent.GetSpecies(6));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"),
                   oPlantEvent.GetSpecies(7));
      assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"),
                   oPlantEvent.GetSpecies(8));

      //*********************************************
       //Now:  Remove Hybrid Spruce.
       //*********************************************
      String[] sSpecies = new String[] {
          "Western Hemlock", "Western Redcedar", "Amabilis Fir",
          "Subalpine Fir", "Lodgepole Pine", "Trembling Aspen",
          "Black Cottonwood", "Paper Birch"};

      oPop.SetSpeciesNames(sSpecies);

      assertEquals(2, oPlant.mp_oPlantings.size());

      //Number one
      oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(0);
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(0));

      //Number two
      oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(1);
      assertEquals(8, oPlantEvent.GetNumberOfSpecies());
      assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"),
                   oPlantEvent.GetSpecies(0));
      assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"),
                   oPlantEvent.GetSpecies(1));
      assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"),
                   oPlantEvent.GetSpecies(2));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"),
                   oPlantEvent.GetSpecies(3));
      assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"),
                   oPlantEvent.GetSpecies(4));
      assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"),
                   oPlantEvent.GetSpecies(5));
      assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"),
                   oPlantEvent.GetSpecies(6));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"),
                   oPlantEvent.GetSpecies(7));

      //*********************************************
      //Now:  Remove Subalpine Fir and Amabilis Fir, and rearrange the
      //species names.
      //*********************************************
      sSpecies = new String[] {"Western Hemlock", "Black Cottonwood",
              "Western Redcedar", "Lodgepole Pine", "Trembling Aspen",
              "Paper Birch"};

      oPop.SetSpeciesNames(sSpecies);

      assertEquals(1, oPlant.mp_oPlantings.size());

      //Number one
      oPlantEvent = (PlantingData) oPlant.mp_oPlantings.elementAt(0);
      assertEquals(6, oPlantEvent.GetNumberOfSpecies());
      assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"),
                   oPlantEvent.GetSpecies(0));
      assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"),
                   oPlantEvent.GetSpecies(1));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"),
                   oPlantEvent.GetSpecies(2));
      assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"),
                   oPlantEvent.GetSpecies(3));
      assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"),
                   oPlantEvent.GetSpecies(4));
      assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"),
                   oPlantEvent.GetSpecies(5));


    }
    catch (ModelException oErr) {
      fail("XML tree map reading failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * This writes an XML file to test value setting.
   * @return Name of file.
   * @throws ModelException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 24, 2004: Created (LEM)
   */
  protected String WriteXMLFile1() throws ModelException {
    try {
      String sFileName = "\\loratestxml1.xml";
      java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

      oOut.write(
          "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
      oOut.write("<paramFile fileCode=\"06010101\">");

      //Plot
      oOut.write("<plot>");
      oOut.write("<plot_lenX>200.0</plot_lenX>");
      oOut.write("<plot_lenY>200.0</plot_lenY>");
      oOut.write("<plot_latitude>55.37</plot_latitude>");
      oOut.write("</plot>");

      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"Western_Hemlock\"/>");
      oOut.write("<tr_species speciesName=\"Western_Redcedar\"/>");
      oOut.write("<tr_species speciesName=\"Amabilis_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Subalpine_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Hybrid_Spruce\"/>");
      oOut.write("<tr_species speciesName=\"Lodgepole_Pine\"/>");
      oOut.write("<tr_species speciesName=\"Trembling_Aspen\"/>");
      oOut.write("<tr_species speciesName=\"Black_Cottonwood\"/>");
      oOut.write("<tr_species speciesName=\"Paper_Birch\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("</trees>");

      //Include an empty behavior list - this triggers setup
      oOut.write("<behaviorList>");
      oOut.write("</behaviorList>");

      oOut.write("<plant>");

      //Initial diam10s
      oOut.write("<pl_initialDiam10>");
      oOut.write("<pl_idVal species=\"Hybrid_Spruce\">0.5</pl_idVal>");
      oOut.write("<pl_idVal species=\"Lodgepole_Pine\">0.6</pl_idVal>");
      oOut.write("<pl_idVal species=\"Trembling_Aspen\">0.7</pl_idVal>");
      oOut.write("<pl_idVal species=\"Black_Cottonwood\">0.8</pl_idVal>");
      oOut.write("<pl_idVal species=\"Western_Hemlock\">0.1</pl_idVal>");
      oOut.write("<pl_idVal species=\"Western_Redcedar\">0.2</pl_idVal>");
      oOut.write("<pl_idVal species=\"Amabilis_Fir\">0.3</pl_idVal>");
      oOut.write("<pl_idVal species=\"Subalpine_Fir\">0.4</pl_idVal> ");
      oOut.write("<pl_idVal species=\"Paper_Birch\">0.9</pl_idVal> ");
      oOut.write("</pl_initialDiam10>");

      //Plant event:  plant type of gridded, one species, one cell.
      oOut.write("<pl_plantEvent>");
      oOut.write("<pl_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<pl_timestep>1</pl_timestep>");
      oOut.write("<pl_spaceType>gridded</pl_spaceType>");
      oOut.write("<pl_distanceOrDensity>3.2</pl_distanceOrDensity>");
      oOut.write("<pl_amountToPlant>");
      oOut.write("<pl_atpVal species=\"Subalpine_Fir\">100</pl_atpVal> ");
      oOut.write("</pl_amountToPlant>");
      oOut.write("<pl_applyToCell x=\"6\" y=\"7\" />");
      oOut.write("</pl_plantEvent>");

      //Plant event:  plant type of random.  All species.  Many cells.
      oOut.write("<pl_plantEvent>");
      oOut.write("<pl_applyToSpecies species=\"Amabilis_Fir\" /> ");
      oOut.write("<pl_applyToSpecies species=\"Black_Cottonwood\" /> ");
      oOut.write("<pl_applyToSpecies species=\"Lodgepole_Pine\" /> ");
      oOut.write("<pl_applyToSpecies species=\"Western_Redcedar\" />");
      oOut.write("<pl_applyToSpecies species=\"Trembling_Aspen\" />");
      oOut.write("<pl_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<pl_applyToSpecies species=\"Paper_Birch\" />");
      oOut.write("<pl_applyToSpecies species=\"Western_Hemlock\" />");
      oOut.write("<pl_applyToSpecies species=\"Hybrid_Spruce\" />");
      oOut.write("<pl_timestep>2</pl_timestep>");
      oOut.write("<pl_spaceType>random</pl_spaceType>");
      oOut.write("<pl_distanceOrDensity>40</pl_distanceOrDensity>");
      oOut.write("<pl_amountToPlant>");
      oOut.write("<pl_atpVal species=\"Western_Redcedar\">0</pl_atpVal>");
      oOut.write("<pl_atpVal species=\"Trembling_Aspen\">10</pl_atpVal>");
      oOut.write("<pl_atpVal species=\"Subalpine_Fir\">11</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Paper_Birch\">12</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Amabilis_Fir\">3</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Black_Cottonwood\">16</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Lodgepole_Pine\">21</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Western_Hemlock\">8</pl_atpVal> ");
      oOut.write("<pl_atpVal species=\"Hybrid_Spruce\">19</pl_atpVal> ");
      oOut.write("</pl_amountToPlant>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"6\"/> ");
      oOut.write("<pl_applyToCell x=\"8\" y=\"7\"/> ");
      oOut.write("<pl_applyToCell x=\"8\" y=\"8\"/> ");
      oOut.write("<pl_applyToCell x=\"8\" y=\"9\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"10\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"11\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"12\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"13\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"14\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"15\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"16\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"17\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"18\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"19\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"20\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"21\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"22\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"23\"/>");
      oOut.write("<pl_applyToCell x=\"8\" y=\"24\"/>");
      oOut.write("</pl_plantEvent>");
      oOut.write("</plant>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

}
